@@ -1,4 +1,4 @@
-/* $Id: gdict-app.c,v 1.45 2002/04/26 14:51:47 gman Exp $ */
+/* $Id: gdict-app.c,v 1.55 2002/05/13 11:35:56 gman Exp $ */
 /* -*- mode: c; style: k&r; c-basic-offset: 4 -*- */
 
 /*
@@ -40,22 +40,30 @@
 
 #define APPNAME "gnome-dictionary"
 
+#define GDICT_RESPONSE_FIND 100
+
 GtkWidget *gdict_app;
 GtkWidget *gdict_appbar;
 GtkWidget *gnome_word_entry;
 GtkWidget *word_entry;
+GtkWidget *find_entry;
 GDictDefbox *defbox;
 GDictSpeller *speller;
 GtkWidget *pref_dialog;
 GtkWidget *socket_error_dialog;
 gboolean  gail_loaded = FALSE;
 
+static gchar *find_text = NULL;
+static gboolean search_from_beginning = TRUE;
+
 #ifdef HAVE_GNOME_PRINT
 GnomePrinter *gdict_printer = NULL;
 #endif /* HAVE_GNOME_PRINT */
 
 dict_context_t *context;
 
+static void gdict_edit_menu_set_sensitivity (gboolean flag);
+
 static gint
 socket_dialog_close_cb (GtkWidget *widget, gpointer data) 
 {
@@ -74,6 +82,7 @@ socket_error_cb (GtkWidget *widget, gchar *message, gpointer data)
     
 }
 
+
 /* gdict_init_context
  *
  * Initialises the context object with information on the current server
@@ -104,7 +113,7 @@ gdict_app_clear (void)
     gchar *word;
     
     gdict_defbox_clear (defbox);
-  
+
     /* Update entry */
     if ((word = gdict_defbox_get_word (defbox))) {
         gtk_entry_set_text (GTK_ENTRY(word_entry), word);
@@ -255,6 +264,7 @@ def_lookup_done_cb (GtkWidget *widget, gpointer data)
     gnome_appbar_pop (GNOME_APPBAR (gdict_appbar));
     gnome_appbar_push (GNOME_APPBAR (gdict_appbar), _("Lookup done"));
     gnome_appbar_refresh (GNOME_APPBAR (gdict_appbar));
+    gdict_edit_menu_set_sensitivity (TRUE);
 }
 
 static void
@@ -263,7 +273,8 @@ def_not_found_cb (GtkWidget *widget, gpointer data)
     gnome_appbar_pop (GNOME_APPBAR (gdict_appbar));
     gnome_appbar_push (GNOME_APPBAR (gdict_appbar), _("No matches found"));
     gnome_appbar_refresh (GNOME_APPBAR (gdict_appbar));
-    
+    gdict_edit_menu_set_sensitivity (FALSE);
+ 
     if (gdict_pref.smart) {
         gdict_spell (gdict_defbox_get_word (defbox), FALSE);
     }
@@ -343,6 +354,8 @@ close_cb (GtkMenuItem *menuitem, gpointer user_data)
 static void
 exit_cb (GtkMenuItem *menuitem, gpointer user_data) 
 {
+    if (find_text)
+    	g_free (find_text);
     gtk_main_quit();
 }
 
@@ -370,6 +383,7 @@ paste_cb (GtkWidget *menuitem, gpointer user_data)
 static void
 clear_cb (GtkWidget *menuitem, gpointer user_data) 
 {
+    gdict_edit_menu_set_sensitivity (FALSE);  
     gdict_defbox_clear (defbox);
 }
 
@@ -390,48 +404,117 @@ select_all_cb (GtkMenuItem *menuitem, gpointer user_data)
 
 }
 
-static gchar *find_text;
 
 static void
-prompt_response_cb (GnomeAppBar *ab) 
+find_word_dialog_cb (GtkDialog *dialog, gint id, gpointer data)
 {
-    gchar *tmp;
+     GtkEntry *entry;
+     const gchar *str;
 
-    tmp = gnome_appbar_get_response(ab);
-    /* fprintf(stderr, "Response\n"); */
-    if (tmp) {
-        g_free(find_text);
-        find_text = gnome_appbar_get_response(ab);
-        if (find_text)
-            gdict_defbox_find (defbox, find_text, TRUE);
-        gnome_appbar_clear_prompt(GNOME_APPBAR(gdict_appbar));
-	g_free(tmp);
-    }
+     entry = GTK_ENTRY (data);
+    
+     gnome_appbar_pop (GNOME_APPBAR (gdict_appbar));
+
+     switch (id) {
+        case GTK_RESPONSE_DELETE_EVENT:
+        case GTK_RESPONSE_CLOSE:
+		gtk_widget_destroy (GTK_WIDGET (dialog));
+		break;
+	case GDICT_RESPONSE_FIND:
+		str = gtk_entry_get_text (entry);
+
+		if (str == NULL)
+			return;
+			
+		if (gdict_defbox_find (defbox, str, search_from_beginning))
+		{
+			search_from_beginning = FALSE;
+			
+			if (find_text != NULL)
+				g_free (find_text);
+			
+			find_text = g_strdup (str);
+		}
+		else
+			search_from_beginning = TRUE;
+		
+		break;
+     }
 }
 
 static void
 find_cb (GtkMenuItem *menuitem, gpointer user_data) 
 {
-    if (!defbox->def_cmd)
-        return;
-    gnome_appbar_pop (GNOME_APPBAR (gdict_appbar));
-    gnome_appbar_set_prompt (GNOME_APPBAR (gdict_appbar), _("Find:"), TRUE);
+    static GtkWidget *dialog = NULL;
+    static GtkWidget *entry = NULL;
+    GtkWidget *hbox;
+    GtkWidget *label;
+    
+    if (dialog == NULL)
+    {
+    	dialog = gtk_dialog_new_with_buttons (_("Find"),
+					  GTK_WINDOW (gdict_app),
+					  GTK_DIALOG_DESTROY_WITH_PARENT,
+					  GTK_STOCK_CLOSE,
+					  GTK_RESPONSE_CLOSE,
+					  NULL);
+    	hbox = gtk_hbox_new (FALSE, 8);
+	
+    	label = gtk_label_new_with_mnemonic ("_Search for:");
+    
+    	entry = gtk_entry_new ();
+   	gtk_entry_set_activates_default (GTK_ENTRY (entry), TRUE);
+    
+	gtk_label_set_mnemonic_widget (GTK_LABEL (label), entry);
+
+    	gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 0);
+    	gtk_box_pack_start (GTK_BOX (hbox), entry, TRUE, TRUE, 0);
+
+    	gtk_container_set_border_width (GTK_CONTAINER (hbox), 6);
+    
+    	gtk_container_add (GTK_CONTAINER (GTK_DIALOG (dialog)->vbox), hbox);
+    	gdict_dialog_add_button (GTK_DIALOG (dialog), _("_Find"), 
+		    	GTK_STOCK_FIND, GDICT_RESPONSE_FIND);
+    
+    	gtk_dialog_set_default_response (GTK_DIALOG (dialog), GDICT_RESPONSE_FIND);
+		    
+   	g_signal_connect (G_OBJECT (dialog), "response", 
+		      	  G_CALLBACK (find_word_dialog_cb), entry);
+
+	g_signal_connect(G_OBJECT (dialog), "destroy",
+			 G_CALLBACK (gtk_widget_destroyed), &dialog);
+
+	gtk_window_set_resizable (GTK_WINDOW (dialog), FALSE);
+
+	gtk_widget_show_all (GTK_WIDGET (dialog));
+
+	search_from_beginning = TRUE;
+    }
+    else
+	gtk_window_present (GTK_WINDOW (dialog));
+
+    if (find_text != NULL)
+	    gtk_entry_set_text (GTK_ENTRY (entry), find_text);
+
+    gtk_entry_select_region (GTK_ENTRY (entry), 0, -1);
+    gtk_widget_grab_focus (entry);
 }
 
 static void
 find_again_cb (GtkMenuItem *menuitem, gpointer user_data) 
 {
-    if (!defbox->def_cmd)
-        return;
     if (find_text)
-        gdict_defbox_find (defbox, find_text, FALSE);
+        search_from_beginning = !gdict_defbox_find (defbox, find_text, search_from_beginning);
+    else
+	find_cb (NULL, NULL);
 }
 
 static void
 pref_dialog_apply_cb (GtkWidget *widget, gpointer user_data) 
 {
     gdict_defbox_reset (defbox, context);
-    if (speller) gdict_speller_reset (speller, context);
+    if (speller) 
+	gdict_speller_reset (speller, context);
 }
 
 void
@@ -530,11 +613,81 @@ add_atk_relation(GtkWidget *obj1, GtkWidget *obj2, AtkRelationType rel_type)
 
 }
 
+GtkWidget*
+gdict_dialog_add_button (GtkDialog *dialog, const gchar* text, const gchar* stock_id, 
+                         gint response_id)
+{
+        GtkWidget *button;
+
+        g_return_val_if_fail (GTK_IS_DIALOG (dialog), NULL);
+        g_return_val_if_fail (text != NULL, NULL);
+        g_return_val_if_fail (stock_id != NULL, NULL);
+
+        button = gdict_button_new_with_stock_image (text, stock_id);
+        g_return_val_if_fail (button != NULL, NULL);
+
+        GTK_WIDGET_SET_FLAGS (button, GTK_CAN_DEFAULT);
+
+        gtk_widget_show (button);
+
+        gtk_dialog_add_action_widget (dialog, button, response_id);
+
+        return button;
+}
+
+GtkWidget* 
+gdict_button_new_with_stock_image (const gchar* text, const gchar* stock_id)
+{
+	GtkWidget *button;
+	GtkStockItem item;
+	GtkWidget *label;
+	GtkWidget *image;
+	GtkWidget *hbox;
+	GtkWidget *align;
+
+	button = gtk_button_new ();
+
+ 	if (GTK_BIN (button)->child)
+    		gtk_container_remove (GTK_CONTAINER (button),
+				      GTK_BIN (button)->child);
+
+  	if (gtk_stock_lookup (stock_id, &item))
+    	{
+      		label = gtk_label_new_with_mnemonic (text);
+
+		gtk_label_set_mnemonic_widget (GTK_LABEL (label), GTK_WIDGET (button));
+      
+		image = gtk_image_new_from_stock (stock_id, GTK_ICON_SIZE_BUTTON);
+      		hbox = gtk_hbox_new (FALSE, 2);
+
+      		align = gtk_alignment_new (0.5, 0.5, 0.0, 0.0);
+      
+      		gtk_box_pack_start (GTK_BOX (hbox), image, FALSE, FALSE, 0);
+      		gtk_box_pack_end (GTK_BOX (hbox), label, FALSE, FALSE, 0);
+      
+      		gtk_container_add (GTK_CONTAINER (button), align);
+      		gtk_container_add (GTK_CONTAINER (align), hbox);
+      		gtk_widget_show_all (align);
+
+      		return button;
+    	}
+
+      	label = gtk_label_new_with_mnemonic (text);
+      	gtk_label_set_mnemonic_widget (GTK_LABEL (label), GTK_WIDGET (button));
+  
+  	gtk_misc_set_alignment (GTK_MISC (label), 0.5, 0.5);
+
+  	gtk_widget_show (label);
+  	gtk_container_add (GTK_CONTAINER (button), label);
+
+	return button;
+}
+
 static GnomeUIInfo dictionary_menu_uiinfo[] = {
-    GNOMEUIINFO_ITEM_STOCK (N_("_Lookup"),
+    GNOMEUIINFO_ITEM_STOCK (N_("_Look Up Word"),
 			    N_("Lookup word in dictionary"),
 			    lookup_cb, GNOME_STOCK_MENU_SEARCH),
-    GNOMEUIINFO_ITEM_STOCK (N_("_Spell"), 
+    GNOMEUIINFO_ITEM_STOCK (N_("Check _Spelling"), 
 			    N_("Check word spelling"), 
 			    spell_cb, GNOME_STOCK_MENU_SPELLCHECK),
 #ifdef HAVE_GNOME_PRINT
@@ -543,9 +696,7 @@ static GnomeUIInfo dictionary_menu_uiinfo[] = {
     GNOMEUIINFO_MENU_PRINT_SETUP_ITEM (print_setup_cb, NULL),
 #endif /* HAVE_GNOME_PRINT */
     GNOMEUIINFO_SEPARATOR,
-    GNOMEUIINFO_ITEM_STOCK (N_("_Quit"),
-			    N_("Quit dictionary"),
-			    exit_cb, GTK_STOCK_QUIT),
+    GNOMEUIINFO_MENU_QUIT_ITEM(exit_cb, NULL),
     GNOMEUIINFO_END
 };
 
@@ -618,6 +769,7 @@ GtkWidget *gdict_app_create (gboolean applet) {
     tooltips = gtk_tooltips_new ();
   
     gdict_app = gnome_app_new ("gnome-dictionary", _("Dictionary"));
+
     gtk_window_set_default_size (GTK_WINDOW (gdict_app), 450, 330);
     
     dock = GNOME_APP (gdict_app)->dock;
@@ -627,27 +779,23 @@ GtkWidget *gdict_app_create (gboolean applet) {
         memcpy (&dictionary_menu_uiinfo[EXIT_FILE_MENU_ITEM],
 		&applet_close_item, sizeof(GnomeUIInfo));
 
-    gnome_app_create_menus (GNOME_APP (gdict_app), menubar_uiinfo);
+    gnome_app_create_menus (GNOME_APP (gdict_app), menubar_uiinfo); 
+    gdict_edit_menu_set_sensitivity (FALSE);  
     
     vbox = gtk_vbox_new (FALSE, 0);
     gtk_widget_show (vbox);
     gnome_app_set_contents (GNOME_APP (gdict_app), vbox);
     
-    hbox = gtk_hbox_new (FALSE, GNOME_PAD_SMALL);
-    gtk_container_set_border_width (GTK_CONTAINER (hbox), GNOME_PAD_SMALL);
+    hbox = gtk_hbox_new (FALSE, 6);
+    gtk_container_set_border_width (GTK_CONTAINER (hbox), 6);
     gtk_widget_show (hbox);
     gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, FALSE, 0);
     
-    word_label = gtk_label_new_with_mnemonic (_("_Word : "));
+    word_label = gtk_label_new_with_mnemonic (_("_Word: "));
     gtk_widget_show ( GTK_WIDGET(word_label) );
     gtk_box_pack_start (GTK_BOX (hbox), word_label, FALSE, FALSE, 0);
     
-    label = gtk_label_new_with_mnemonic (_("_Look Up"));
-    gtk_widget_show (label);
-    gtk_misc_set_padding (GTK_MISC (label), 2, 1);
-    
-    button = gtk_button_new ();
-    gtk_container_add (GTK_CONTAINER (button), label);
+    button = gdict_button_new_with_stock_image (_("_Look Up Word"), GTK_STOCK_FIND);
     gtk_widget_show (button);
     gtk_box_pack_end (GTK_BOX (hbox), button, FALSE, FALSE, 0);
     gtk_drag_dest_set (button, GTK_DEST_DEFAULT_ALL, drop_targets, 
@@ -680,8 +828,10 @@ GtkWidget *gdict_app_create (gboolean applet) {
     scrolled = gtk_scrolled_window_new (NULL, NULL);
     gtk_container_set_border_width (GTK_CONTAINER (scrolled), GNOME_PAD_SMALL);    
     gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolled),
-                                        GTK_POLICY_AUTOMATIC,
-                                        GTK_POLICY_AUTOMATIC);
+                                        GTK_POLICY_AUTOMATIC,             
+	       				GTK_POLICY_AUTOMATIC);
+    gtk_scrolled_window_set_shadow_type (GTK_SCROLLED_WINDOW (scrolled), GTK_SHADOW_ETCHED_IN);
+
     gtk_widget_show (scrolled);
   
     defbox = GDICT_DEFBOX (gdict_defbox_new ());
@@ -704,9 +854,11 @@ GtkWidget *gdict_app_create (gboolean applet) {
 
     gtk_box_pack_start (GTK_BOX (vbox), scrolled, TRUE, TRUE, 0);
 
-    gdict_appbar = gnome_appbar_new (FALSE, TRUE, GNOME_PREFERENCES_ALWAYS);
-    gtk_widget_show (gdict_appbar);
+    gdict_appbar = gnome_appbar_new (TRUE, TRUE, GNOME_PREFERENCES_NEVER);
     gnome_app_set_statusbar (GNOME_APP (gdict_app), gdict_appbar);
+    gnome_app_install_menu_hints (GNOME_APP (gdict_app), menubar_uiinfo);
+
+    gtk_widget_show (gdict_appbar);
 
     if (!applet)
       gtk_signal_connect (GTK_OBJECT (gdict_app), "delete_event",
@@ -717,8 +869,19 @@ GtkWidget *gdict_app_create (gboolean applet) {
     
     gtk_signal_connect (GTK_OBJECT (word_entry), "activate",
                         GTK_SIGNAL_FUNC(lookup_cb), NULL);
-    gtk_signal_connect (GTK_OBJECT (gdict_appbar), "user_response",
-                        GTK_SIGNAL_FUNC(prompt_response_cb), NULL);
-    
     return gdict_app;
 }
+
+static void
+gdict_edit_menu_set_sensitivity (gboolean flag)
+{
+  static gboolean sensitivity = TRUE;
+  gint i, items[4] = {3, 4, 6, 7};
+
+  if (! (flag ^ sensitivity))
+    return; 
+  for (i=0; i < 4; i++) {
+    sensitivity = flag;
+    gtk_widget_set_sensitive (GTK_WIDGET (edit_menu_uiinfo[items[i]].widget), flag);
+  }
+}
