@@ -1,4 +1,4 @@
-/* $Id: gdict-applet.c,v 1.32 2002/04/25 12:40:48 gman Exp $ */
+/* $Id: gdict-applet.c,v 1.34 2002/05/10 18:18:29 gman Exp $ */
 
 /*
  *  Papadimitriou Spiros <spapadim+@cs.cmu.edu>
@@ -36,118 +36,81 @@
 #define TALL_APPLET_HEIGHT 44
 
 
-static void
-redraw_pixbuf (GDictApplet *applet)
-{
-	GdkPixbuf *tmp;
-	gint width, height;
-	float ratio;
-	gint nwidth, nheight;
-	
-	width = gdk_pixbuf_get_width (applet->pixbuf);
-	height = gdk_pixbuf_get_height (applet->pixbuf);
-	
-	switch (applet->orient) {
-	case PANEL_APPLET_ORIENT_LEFT:
-	case PANEL_APPLET_ORIENT_RIGHT:	
-		ratio = (float)height / (float)width;
-		nwidth = applet->panel_size - 10;
-		nheight = ratio * applet->panel_size - 10;
-		break;
-	case PANEL_APPLET_ORIENT_UP:
-	case PANEL_APPLET_ORIENT_DOWN:
-		ratio = (float)width / (float)height;
-		nheight = applet->panel_size - 10;
-		nwidth = ratio * applet->panel_size - 10;
-		break;
-	}
-
-	nwidth = CLAMP (nwidth, 10, width);
-	nheight = CLAMP (nheight, 10, height);
-	
-	tmp = gdk_pixbuf_scale_simple (applet->pixbuf, nwidth, nheight, GDK_INTERP_HYPER);
-	
-	if (applet->image) {
-		gtk_widget_destroy (applet->image);
-	}
-		
-	applet->image = gtk_image_new_from_pixbuf (tmp);
-	gtk_widget_show (applet->image);
-	gtk_container_add (GTK_CONTAINER (applet->button_widget), applet->image);
-	gdk_pixbuf_unref (tmp);
-}
-
 static void
 change_size_cb (PanelApplet *widget, gint size, GDictApplet *applet)
 {
-
 	applet->panel_size = size;
-	redraw_pixbuf (applet);	
 
 }
 
 static void
 change_orient_cb (PanelApplet *widget, PanelAppletOrient orient, GDictApplet *applet)
 {
-
 	applet->orient = orient;
-	redraw_pixbuf (applet);
 	
 }
 
-#if 0
 static void
 entry_activate_cb (GtkEntry *entry, gpointer data)
 {
-    gchar *text = gtk_editable_get_chars (GTK_EDITABLE (entry), 0, -1);
+    	gchar *text = gtk_editable_get_chars (GTK_EDITABLE (entry), 0, -1);
     
-    if (!context)
+    	if (!context)
 		gdict_init_context ();
-    if (!GTK_WIDGET_VISIBLE (gdict_app))
-        gtk_widget_show (gdict_app);
+    	if (!GTK_WIDGET_VISIBLE (gdict_app))
+        	gtk_widget_show (gdict_app);
     
-    if (*text == 0)
-        return;
-    g_strdown (text);
-    gdict_app_do_lookup (text);
-    gtk_entry_set_text (GTK_ENTRY (word_entry), text);
-    g_free (text);
+    	gtk_window_present (GTK_WINDOW (gdict_app));
+    
+    	if (!text || *text == 0)
+        	return;
+        
+    	g_strdown (text);
+    	gdict_app_do_lookup (text);
+    	gtk_entry_set_text (GTK_ENTRY (word_entry), text);
+    	g_free (text);
  
- if 0   
-    gtk_entry_set_text (GTK_ENTRY (applet->entry_widget), "");
-    gtk_entry_set_text (GTK_ENTRY (word_entry), text);
-    gtk_editable_select_region (GTK_EDITABLE (word_entry), 0, strlen (text));
-    gdict_app_do_lookup (text);
- endif
+
+}
+
+static void
+lookup_button_drag_cb (GtkWidget *widget, GdkDragContext *context, gint x, gint y,
+		       GtkSelectionData *sd, guint info, guint t, gpointer data)
+{
+	GDictApplet *applet = data;
+	gchar *text;
+	
+	text = gtk_selection_data_get_text (sd);
+	
+	if (text) {		
+    		gtk_entry_set_text (GTK_ENTRY (applet->entry_widget), text);
+		entry_activate_cb (GTK_ENTRY (applet->entry_widget), applet);
+    		g_free (text);
+	}
+	
 }
-#endif
 
 static void
 text_received (GtkClipboard *clipboard, const gchar *text, gpointer data)
 {
+	GDictApplet *applet = data;
 	gchar *lookup_text;
 	
-	if (!GTK_WIDGET_VISIBLE (gdict_app))
-        	gtk_widget_show (gdict_app);
-        	
-	gtk_window_present (GTK_WINDOW (gdict_app));
-
 	if (!text)
 		return;
-
-	lookup_text = g_strstrip (text);
-#if 0
-	g_print ("text = %s \n", lookup_text);
-#endif
-
-	if (!context)
-		gdict_init_context ();
-
-    	if (*lookup_text == 0)
+	
+	lookup_text = g_strdup (text);
+	g_strstrip (lookup_text);
+	
+	if (*lookup_text == 0)
         	return;
     	g_strdown (lookup_text);
+    	gtk_entry_set_text (GTK_ENTRY (applet->entry_widget), text);
+    	entry_activate_cb (GTK_ENTRY (applet->entry_widget), applet);
+#if 0    	
     	gdict_app_do_lookup (lookup_text);
     	gtk_entry_set_text (GTK_ENTRY (word_entry), lookup_text);
+#endif
     	g_free (lookup_text);
  
 	
@@ -157,11 +120,8 @@ static void
 button_press_cb (GtkButton *button, gpointer data)
 {
 	GDictApplet *applet = data;
-	GtkClipboard *clipboard;
-	
-	clipboard = gtk_clipboard_get (GDK_SELECTION_PRIMARY);
- 	gtk_clipboard_request_text (clipboard, text_received, NULL);
 	
+	entry_activate_cb (GTK_ENTRY (applet->entry_widget), applet);
 	
 }
 
@@ -180,7 +140,16 @@ button_press_hack (GtkWidget *w, GdkEventButton *event, gpointer data)
     
 }
 
+static void
+lookup_selected_text (BonoboUIComponent *uic, gpointer data, const gchar *verbname)
+{
+	GDictApplet *applet = data;
+	GtkClipboard *clipboard;
 
+	clipboard = gtk_clipboard_get (GDK_SELECTION_PRIMARY);
+ 	gtk_clipboard_request_text (clipboard, text_received, applet);
+ 	
+}
 static void
 about_cb (BonoboUIComponent *uic,
           gpointer           user_data,
@@ -240,6 +209,7 @@ spell_cb (BonoboUIComponent *uic,
 
 static const BonoboUIVerb gdict_applet_menu_verbs [] = {
 	BONOBO_UI_VERB ("def win", show_hide_defbox_cb),
+	BONOBO_UI_VERB ("sel", lookup_selected_text),
         BONOBO_UI_VERB ("prefs", prefs_cb),
         BONOBO_UI_VERB ("help", help_cb),
         BONOBO_UI_VERB ("about", about_cb),
@@ -252,32 +222,33 @@ gdict_applet_new (PanelApplet *parent_applet)
 {
 	GDictApplet *applet;
  	GtkWidget *hbox;
- 	GtkTooltips *tooltips;
+ 	static GtkTargetEntry drop_targets [] = {
+    		{ "UTF8_STRING", 0, 0 },
+  		{ "COMPOUND_TEXT", 0, 0 },
+  		{ "TEXT", 0, 0 },
+  		{ "text/plain", 0, 0 },
+  		{ "STRING",     0, 0 }
+    	};
  	
  	applet = g_new0 (GDictApplet, 1);
  	
+ 	gnome_window_icon_set_default_from_file (GNOME_ICONDIR"/gdict.png");
+ 	
 	hbox = gtk_hbox_new (FALSE, 0);
-	
-	applet->pixbuf = gdk_pixbuf_new_from_file (GNOME_ICONDIR"/gdict.png", NULL);
-	
-	applet->button_widget = gtk_button_new ();
 
+	applet->button_widget = gtk_button_new_with_mnemonic (_("_Lookup"));
+	gtk_drag_dest_set (applet->button_widget, GTK_DEST_DEFAULT_ALL, drop_targets, 
+    		           G_N_ELEMENTS (drop_targets), GDK_ACTION_COPY | GDK_ACTION_MOVE);
+	gtk_box_pack_start (GTK_BOX (hbox), applet->button_widget, FALSE, FALSE, 2);
+	
         if (GTK_IS_ACCESSIBLE (gtk_widget_get_accessible (applet->button_widget)))          
         add_atk_namedesc (GTK_WIDGET (applet->button_widget), _("Dictionary Applet"), NULL);
 
-	gtk_box_pack_start (GTK_BOX (hbox), applet->button_widget, FALSE, FALSE, 2);
-	gtk_widget_show_all (hbox);
-
-#if 0	  	
- 	/*applet->entry_widget = gtk_entry_new ();
- 	gtk_box_pack_start (GTK_BOX (hbox), applet->entry_widget, TRUE, TRUE, 0);*/
-#endif 	
-
- 	tooltips = gtk_tooltips_new ();
- 	gtk_tooltips_set_tip (tooltips, applet->button_widget,
- 			      _("Press button to look up any highlighted text in a dictionary"),
- 			      NULL);
+	applet->entry_widget = gtk_entry_new ();
+ 	gtk_box_pack_start (GTK_BOX (hbox), applet->entry_widget, TRUE, TRUE, 0);
  	
+ 	gtk_widget_show_all (hbox);
+	  	
  	applet->applet_widget = GTK_WIDGET(parent_applet);
  	gtk_container_add(GTK_CONTAINER(parent_applet), hbox);
  	applet->panel_size = panel_applet_get_size (PANEL_APPLET (applet->applet_widget));
@@ -295,18 +266,18 @@ gdict_applet_new (PanelApplet *parent_applet)
  	gtk_widget_show (applet->applet_widget);
  	gdict_pref_load ();
  	
- 	redraw_pixbuf (applet);	
- 	
  	/* server will be contacted when an action is performed */
  	context = NULL;
-#if 0 	
- 	/*g_signal_connect (G_OBJECT (applet->entry_widget), "activate",
- 			  G_CALLBACK (entry_activate_cb), applet);*/
-#endif
+	
+ 	g_signal_connect (G_OBJECT (applet->entry_widget), "activate",
+ 			  G_CALLBACK (entry_activate_cb), applet);
+
 	g_signal_connect (G_OBJECT (applet->button_widget), "button_press_event",
 			  G_CALLBACK (button_press_hack), parent_applet);
  	g_signal_connect (G_OBJECT (applet->button_widget), "clicked",
  			  G_CALLBACK (button_press_cb), applet);
+ 	g_signal_connect (G_OBJECT (applet->button_widget), "drag_data_received",
+    		          G_CALLBACK (lookup_button_drag_cb), applet);
 
  	g_signal_connect (G_OBJECT (applet->applet_widget), "change_size",
  			  G_CALLBACK (change_size_cb), applet);
