@@ -1,4 +1,4 @@
-/* $Id: gdict-pref-dialog.c,v 1.23 2002/04/04 03:38:26 kevinv Exp $ */
+/* $Id: gdict-pref-dialog.c,v 1.30 2002/05/11 21:04:27 gman Exp $ */
 
 /*
  *  Mike Hughes <mfh@psilord.com>
@@ -48,8 +48,12 @@ static gchar *typeface_sel_names[NUM_TYPEFACES] = {
 #define NUM_TYPEFACE_COLS 2
 
 GtkWidget *db_label, *strat_label;
+GtkWidget *error_label1 = NULL;
+GtkWidget *error_label2 = NULL;
+GtkWidget *alignment1 = NULL;
+GtkWidget *alignment2 = NULL;
+
 static gint gdict_pref_dialog_signals[LAST_SIGNAL] = { 0 };
-extern GConfClient *client;
 
 static void gdict_pref_dialog_init (GDictPrefDialog *pref_dialog);
 static void gdict_pref_dialog_class_init (GDictPrefDialogClass *class);
@@ -85,6 +89,7 @@ static void set_default_server      (GtkButton *button, gpointer data);
 static void set_default_port        (GtkButton *button, gpointer data);
 static void smart_lookup_cb	    (GtkToggleButton *toggle, gpointer data);
 
+
 /* gdict_pref_dialog_get_type
  *
  * Register the GDictPrefDialog type with Gtk's type system if necessary and
@@ -177,7 +182,7 @@ create_dialog (GDictPrefDialog *pref_dialog)
     pref_dialog->database = gdict_pref.database;
     pref_dialog->dfl_strat = gdict_pref.dfl_strat;
     
-    gtk_window_set_title (GTK_WINDOW (pref_dialog), _("Preferences"));
+    gtk_window_set_title (GTK_WINDOW (pref_dialog), _("Dictionary Preferences"));
     gtk_window_set_policy (GTK_WINDOW (pref_dialog), FALSE, FALSE, FALSE);    
     vbox = GTK_DIALOG (pref_dialog)->vbox;
 
@@ -241,12 +246,10 @@ create_dialog (GDictPrefDialog *pref_dialog)
         add_atk_namedesc(GTK_WIDGET(button), NULL, _("Reset port to default"));
         add_atk_namedesc(server_label, _("Server"), _("Server"));
         add_atk_namedesc( GTK_WIDGET (pref_dialog->server_entry), _("Server Entry"), _("Enter the Server Name"));
-        add_atk_relation(server_label, GTK_WIDGET (pref_dialog->server_entry), ATK_RELATION_LABEL_FOR);
         add_atk_relation( GTK_WIDGET (pref_dialog->server_entry), server_label, ATK_RELATION_LABELLED_BY);
 
         add_atk_namedesc(port_label, _("Port"), _("Port"));
         add_atk_namedesc( GTK_WIDGET(pref_dialog->port_entry), _("Port Entry"), _("Enter the Port Number"));
-        add_atk_relation(port_label, GTK_WIDGET(pref_dialog->port_entry), ATK_RELATION_LABEL_FOR);
         add_atk_relation( GTK_WIDGET(pref_dialog->port_entry), port_label, ATK_RELATION_LABELLED_BY);
 
     }
@@ -267,14 +270,18 @@ create_dialog (GDictPrefDialog *pref_dialog)
     
     gtk_widget_show_all (GTK_WIDGET (pref_dialog->table));
 
-
-
     gtk_dialog_add_button (GTK_DIALOG (pref_dialog), GTK_STOCK_CLOSE,
     			   GTK_RESPONSE_CLOSE);  
+
+    gtk_dialog_set_default_response (GTK_DIALOG (pref_dialog), GTK_RESPONSE_CLOSE);
     
     g_signal_connect (G_OBJECT (pref_dialog), "response",
     		      G_CALLBACK (response_cb), NULL);
 
+    /* To prevent dialog desctruction when pressing ESC */
+    g_signal_connect (G_OBJECT (pref_dialog), "delete_event", 
+		      G_CALLBACK (gtk_true), NULL);
+
     if (gdict_pref.server != NULL)
 	    gtk_entry_set_text (GTK_ENTRY (pref_dialog->server_entry),
 				gdict_pref.server);
@@ -284,6 +291,7 @@ create_dialog (GDictPrefDialog *pref_dialog)
     gtk_toggle_button_set_active
 	(GTK_TOGGLE_BUTTON (pref_dialog->smart_lookup_btn), gdict_pref.smart);
 
+    gtk_widget_show_all (GTK_WIDGET (pref_dialog));
 
 }
 
@@ -311,7 +319,6 @@ gdict_pref_dialog_new (dict_context_t *context)
     pref_dialog_reset_db (pref_dialog);
     pref_dialog_reset_strat (pref_dialog);
    
-
     return GTK_WIDGET (pref_dialog);
 }
 
@@ -341,7 +348,7 @@ pref_set_db_cb (GtkWidget *widget, gpointer data)
     pref_dialog->database = 
         gtk_object_get_data (GTK_OBJECT (widget), "db_name");
         
-    gconf_client_set_string (client, "/apps/gdict/database", pref_dialog->database, NULL);
+    gconf_client_set_string (gdict_get_gconf_client (), "/apps/gnome-dictionary/database", pref_dialog->database, NULL);
 }
 
 /* pref_set_strat_cb
@@ -358,7 +365,7 @@ pref_set_strat_cb (GtkWidget *widget, gpointer data)
     pref_dialog->dfl_strat = 
         gtk_object_get_data (GTK_OBJECT (widget), "strat_name");
     
-    gconf_client_set_string (client, "/apps/gdict/stratagy", pref_dialog->dfl_strat, NULL);
+    gconf_client_set_string (gdict_get_gconf_client (), "/apps/gnome-dictionary/strategy", pref_dialog->dfl_strat, NULL);
 }
 
 /* pref_dialog_add_db
@@ -438,8 +445,16 @@ pref_dialog_reset_db (GDictPrefDialog *pref_dialog) {
     pref_dialog->get_db_cmd->status_notify_cb = pref_status_cb;
     pref_dialog->get_db_cmd->user_data = pref_dialog;
     pref_dialog->database_idx = 0;
+
+    if ((alignment1) && (error_label1))
+    {
+	gtk_container_remove (GTK_CONTAINER (alignment1), error_label1);
+	gtk_widget_destroy (GTK_WIDGET (alignment1));
+	error_label1 = NULL;
+	alignment1 = NULL;
+    }
     
-    pref_dialog_add_db (pref_dialog, "!", _("Search all databases"));
+    pref_dialog_add_db (pref_dialog, "*", _("Search all databases"));
  
     if (dict_command_invoke (pref_dialog->get_db_cmd,
 			     pref_dialog->context) == -1)
@@ -447,12 +462,11 @@ pref_dialog_reset_db (GDictPrefDialog *pref_dialog) {
 	/* Could not look up search strategies, so just display a
 	 * label; FIXME: Memory leak
 	 */
-	error_label = gtk_label_new (_("Cannot connect to server"));
-        alignment = gtk_alignment_new (0, 0.5, 0, 0);
-        gtk_container_add (GTK_CONTAINER (alignment), 
-                           GTK_WIDGET (error_label));
-        gtk_table_attach_defaults (pref_dialog->table, alignment, 1, 2, 3, 4);
-        gtk_widget_show_all (GTK_WIDGET (pref_dialog->table));
+	error_label1 = GTK_WIDGET (gtk_label_new (_("Cannot connect to server")));
+	alignment1 = GTK_WIDGET (gtk_alignment_new (0, 0.5, 0, 0));
+	gtk_container_add (GTK_CONTAINER (alignment1), error_label1);
+	gtk_table_attach_defaults (pref_dialog->table, alignment1, 1, 2, 3, 4);
+	gtk_widget_show_all (GTK_WIDGET (pref_dialog->table));
     }
 }
 
@@ -484,6 +498,14 @@ pref_dialog_reset_strat (GDictPrefDialog *pref_dialog) {
     pref_dialog->get_strat_cmd->status_notify_cb = pref_status_cb;
     pref_dialog->get_strat_cmd->user_data = pref_dialog;
     pref_dialog->dfl_strat_idx = 0;
+
+    if ((alignment2) && (error_label2))
+    {
+	gtk_container_remove (GTK_CONTAINER (alignment2), error_label2);
+	gtk_widget_destroy (GTK_WIDGET (alignment2));
+	error_label2 = NULL;
+	alignment2 = NULL;
+    }
     
     if (dict_command_invoke (pref_dialog->get_strat_cmd,
 			     pref_dialog->context) == -1) 
@@ -491,12 +513,11 @@ pref_dialog_reset_strat (GDictPrefDialog *pref_dialog) {
 	/* Could not look up search strategies, so just display a
 	 * label; FIXME: Memory leak
 	 */
-	error_label = gtk_label_new (_("Cannot connect to server"));
-        alignment = gtk_alignment_new (0, 0.5, 0, 0);
-        gtk_container_add (GTK_CONTAINER (alignment), 
-                           GTK_WIDGET (error_label));
-        gtk_table_attach_defaults (pref_dialog->table, alignment, 1, 2, 4, 5);
-        gtk_widget_show_all (GTK_WIDGET (pref_dialog->table));
+	error_label2 = GTK_WIDGET (gtk_label_new (_("Cannot connect to server")));
+	alignment2 = GTK_WIDGET (gtk_alignment_new (0, 0.5, 0, 0));
+	gtk_container_add (GTK_CONTAINER (alignment2), error_label2);
+	gtk_table_attach_defaults (pref_dialog->table, alignment2, 1, 2, 4, 5);
+	gtk_widget_show_all (GTK_WIDGET (pref_dialog->table));
     }
 }
 
@@ -599,7 +620,6 @@ pref_status_cb (dict_command_t *command, DictStatusCode code,
             if ( gail_loaded)
             {
                 add_atk_namedesc(db_label, _("Database"), _("Database Name"));
-                add_atk_relation(db_label, GTK_WIDGET (option_menu), ATK_RELATION_LABEL_FOR);
                 add_atk_relation( GTK_WIDGET (option_menu), db_label, ATK_RELATION_LABELLED_BY);
 
             }
@@ -612,7 +632,6 @@ pref_status_cb (dict_command_t *command, DictStatusCode code,
             if ( gail_loaded )
             {
                 add_atk_namedesc(strat_label, _("Default Strategy"), _("Default Strategy"));
-                add_atk_relation(strat_label, GTK_WIDGET (option_menu), ATK_RELATION_LABEL_FOR);
                 add_atk_relation( GTK_WIDGET (option_menu), strat_label, ATK_RELATION_LABELLED_BY);
 
             }
@@ -624,7 +643,6 @@ static void
 response_cb (GtkDialog *dialog, gint response)
 {
     gtk_widget_hide (GTK_WIDGET (dialog));
-    
 }
 
 static gboolean
@@ -634,8 +652,9 @@ set_server_cb (GtkWidget *widget, GdkEventFocus *event, gpointer data)
     gchar *server;
     server = gtk_editable_get_chars (GTK_EDITABLE (pref_dialog->server_entry), 0, -1);
 
-    if ((server != NULL)
-		    && ((gdict_pref.server == NULL)
+    g_return_val_if_fail (server != NULL, FALSE);
+
+    if ((server != NULL) && ((gdict_pref.server == NULL)
 			    || (strcmp (gdict_pref.server, server)))) {
 	
 	if (gdict_pref.server)
@@ -646,7 +665,7 @@ set_server_cb (GtkWidget *widget, GdkEventFocus *event, gpointer data)
         pref_dialog_reset_db (pref_dialog);
         pref_dialog_reset_strat (pref_dialog);
            
-        gconf_client_set_string (client, "/apps/gdict/server", gdict_pref.server, NULL);
+        gconf_client_set_string (gdict_get_gconf_client (), "/apps/gnome-dictionary/server", gdict_pref.server, NULL);
 
     }
    
@@ -676,7 +695,7 @@ set_port_cb (GtkWidget *widget, GdkEventFocus *event, gpointer data)
         pref_dialog_reset_db (pref_dialog);
         pref_dialog_reset_strat (pref_dialog);
             
-        gconf_client_set_int (client, "/apps/gdict/port", gdict_pref.port, NULL);
+        gconf_client_set_int (gdict_get_gconf_client (), "/apps/gnome-dictionary/port", gdict_pref.port, NULL);
        
     }
     
@@ -716,9 +735,8 @@ smart_lookup_cb	(GtkToggleButton *toggle, gpointer data)
     gboolean toggled;
     
     toggled = gtk_toggle_button_get_active (toggle);
-    
     if (toggled != gdict_pref.smart) {
-        gconf_client_set_bool (client, "/apps/gdict/smart", toggled, NULL);
+        gconf_client_set_bool (gdict_get_gconf_client (), "/apps/gnome-dictionary/smart", toggled, NULL);
     }
     
 
