@@ -22,6 +22,7 @@
 #include <config.h>
 #include <gnome.h>
 #include <locale.h>
+#include <errno.h>
 #include "logview.h"
 #include "logrtns.h"
 
@@ -305,12 +306,25 @@ isLogFile (char *filename)
    fp = fopen (filename, "r");
    if (fp == NULL)
    {
+      /* FIXME: this error message is really quite poor
+       * we should state why the open failed
+       * ie. file too large etc etc..
+       */
       g_snprintf (buff, sizeof (buff),
 		  _("%s could not be opened."), filename);
       ShowErrMessage (buff);
       return FALSE;
    }
-   fgets (buff, sizeof (buff), fp);
+   if (fgets (buff, sizeof (buff), fp) == NULL) {
+       /* Either an error, or empty log */
+       fclose (fp);
+       if (feof(fp)) 
+           /* EOF -> empty file, still a valid log though */
+           return TRUE;
+       else 
+           /* !EOF -> read error */
+           return FALSE;
+   }
    fclose (fp);
    token = strtok (buff, " ");
    /* This is not a good assumption I don't think, especially
@@ -325,6 +339,13 @@ isLogFile (char *filename)
    }
    */
 
+   if (token == NULL)
+   {
+      g_snprintf (buff, sizeof (buff), _("%s not a log file."), filename);
+      ShowErrMessage (buff);
+      return FALSE;
+   }
+
    i = get_month (token);
 
    if (i == 12)
@@ -405,7 +426,8 @@ ReadPageUp (Log * lg, Page * pg)
 {
    LogLine *line;
    FILE *fp;
-   char *c, ch, buffer[R_BUF_SIZE + 1];
+   char *c, buffer[R_BUF_SIZE + 1];
+   int ch;
    int ln;
    long int old_pos;
 
@@ -476,7 +498,8 @@ ReadPageDown (Log * lg, Page * pg, gboolean exec_actions)
 {
    FILE *fp;
    LogLine *line;
-   char *c, ch, buffer[R_BUF_SIZE + 1];
+   char *c, buffer[R_BUF_SIZE + 1];
+   int ch;
    int ln, len;
 
    g_return_val_if_fail (lg != NULL, FALSE);
@@ -1114,6 +1137,8 @@ CloseLog (Log * log)
    Page *cur, *next;
    int i;
 
+   g_return_if_fail (log);
+
    /* Close file */
    if (log->fp != NULL)
       fclose (log->fp);
